﻿using Microsoft.ML;
using System.Data;
using System.Reflection;

namespace EOCS.LogisticRegression.Console
{
    internal class Program
    {
        static void Main(string[] args)
        {
            var ctx = new MLContext();

            // Load data from file
            var path = AppContext.BaseDirectory + "/dataset.csv";
            var data = ctx.Data.LoadFromTextFile<Customer>(path, hasHeader: true, separatorChar: ',');

            // Split data
            var splitData = ctx.Data.TrainTestSplit(data, testFraction: 0.2);

            // Process data
            var categoryTransformer = ctx.Transforms.Categorical.OneHotEncoding(new[]
            { 
                new InputOutputColumnPair("Country", "Country"),
                new InputOutputColumnPair("Gender", "Gender"),
                new InputOutputColumnPair("Tenure", "Tenure"),
                new InputOutputColumnPair("CreditCard", "CreditCard"),
                new InputOutputColumnPair("ActiveMember", "ActiveMember"),
            });
            var normalizeTransformer = ctx.Transforms.NormalizeMinMax(new[]
            {
                new InputOutputColumnPair("CreditScore", "CreditScore"),
                new InputOutputColumnPair("Age", "Age"),
                new InputOutputColumnPair("Balance", "Balance"),
                new InputOutputColumnPair("ProductsNumber", "ProductsNumber"),
                new InputOutputColumnPair("EstimatedSalary", "EstimatedSalary"),
            });
            var featuresTransformer = ctx.Transforms.Concatenate("Features", new[]
            {
                "CreditScore", "Country", "Gender", "Age", "Tenure", "Balance", "ProductsNumber", "CreditCard", "ActiveMember", "EstimatedSalary"
            });

            var dataPipe = categoryTransformer.Append(normalizeTransformer).Append(featuresTransformer).Append(ctx.BinaryClassification.Trainers.LbfgsLogisticRegression());

            var trainedModel = dataPipe.Fit(splitData.TrainSet);           

            // Measure trained model performance            
            var predictions = trainedModel.Transform(splitData.TestSet);

            // Extract model metrics and get accuracy
            var trainedModelMetrics = ctx.BinaryClassification.Evaluate(predictions, labelColumnName: "Label", predictedLabelColumnName: "PredictedLabel");
            var accuracy = trainedModelMetrics.Accuracy;


           

            // Predict an unforeseen input
            var X = new Customer()
            {
                CreditScore = 845,
                Country = "France",
                Gender = "Female",
                Age = 40,
                Tenure = 5,
                Balance = 10,
                ProductsNumber = 1,
                CreditCard = true,
                ActiveMember = true,
                EstimatedSalary = 135000
            };
            

            var pe = ctx.Model.CreatePredictionEngine<Customer, CustomerPrediction>(trainedModel);
            var Y = pe.Predict(X);
            

        }
    }
}
